{
  Vcl IDEInspect Expert

  Created: 13.09.97
  Author:  Martin_Waldenburg
  Version 1.0
  Status: FreeWare
  CopyRight of CompEd by David Spies.
  Except of "Secrets of Delphi 2" by Ray Lischner,
  CompEd is the best sample of using TypeInfo I know.
  This version of CompEd is slightly modified.
  Caution: Using IDEInspect Expert may be dangerous.
           It may shoot down Delphi and Windows too.
  It's provided as is, without a warranty of any kind.
  You use it at your own risc.
}


unit mwIDEInspect;

interface

uses SysUtils,
  Classes,
  Messages,
  Consts,
  Forms,
  Windows,
  Graphics,
  Dialogs,
  ClipBrd,
  Controls,
  EditIntf,
  ExptIntf,
  ToolIntf,
  ExtCtrls,
  Menus,
  comCtrls,
  StdCtrls,
  LibIntf,
  TypInfo,
  WinTypes,
  E_Props,
  E_cmpEd,
  ImageWin,
  ViewImg,
  StrEdit;

type
  TfmIDEInspect = class(TForm)
    TreeView1: TTreeView;
    Panel1: TPanel;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure TreeView1Deletion(Sender: TObject; Node: TTreeNode);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { private declarations }
  public
    { public declarations }
  end;

  TIDEInspectExpert = class(TIExpert)
  private
    MenuItem: TIMenuItemIntf;
  protected
    procedure OnClick( Sender: TIMenuItemIntf); virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function GetName: string; override;
    function GetAuthor: string; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
  end;

procedure Register;

var
  fmIDEInspect: TfmIDEInspect;
  TopNode, Level1Child, Level2Child, Level3Child, Level4Child: TTreenode;

implementation

{$R *.DFM}

procedure Register;
begin
  RegisterLibraryExpert(TIDEInspectExpert.Create);
end;

{ TIDEInspectExpert code }
function TIDEInspectExpert.GetName: String;
begin
  Result := 'IDEInspectExpert'
end;

function TIDEInspectExpert.GetAuthor: String;
begin
  Result := 'Martin_Waldenburg'; { author }
end;

function TIDEInspectExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TIDEInspectExpert.GetIDString: String;
begin
  Result := 'private.IDEInspectExpert';
end;

constructor TIDEInspectExpert.Create;
var
  Main: TIMainMenuIntf;
  ReferenceMenuItem: TIMenuItemIntf;
  Menu: TIMenuItemIntf;
begin
  inherited Create;
  MenuItem := nil;
  if ToolServices <> nil then begin { I'm an expert! }
    Main := ToolServices.GetMainMenu;
    if Main <> nil then begin { we've got the main menu! }
      try 
        { add the menu of your choice }
        ReferenceMenuItem := Main.FindMenuItem('ToolsOptionsItem');
        if ReferenceMenuItem <> nil then
        try
          Menu := ReferenceMenuItem.GetParent;
          if Menu <> nil then
          try
            MenuItem := Menu.InsertItem(ReferenceMenuItem.GetIndex+1,
                              'IDEInspect',
                              'IDEInspectExpertItem','',
                              0,0,0,
                              [mfEnabled, mfVisible], OnClick);
          finally
            Menu.DestroyMenuItem;
          end;
        finally
          ReferenceMenuItem.DestroyMenuItem;
        end;
      finally
        Main.Free;
      end;
    end;
  end;
end;

destructor TIDEInspectExpert.Destroy;
begin
  if MenuItem <> nil then
    MenuItem.DestroyMenuItem;
  inherited Destroy;
end;{Destroy}

procedure TIDEInspectExpert.OnClick( Sender: TIMenuItemIntf);
begin
  if not Assigned(fmIDEInspect) then
  begin
    fmIDEInspect := TfmIDEInspect.Create(Application);
    Application.CreateForm(TCompEditForm, CompEditForm);
    Application.CreateForm(TImageForm, ImageForm);
    Application.CreateForm(TViewImageForm, ViewImageForm);
    Application.CreateForm(TSedForm, SedForm);
  end;
  fmIDEInspect.Show;
  fmIDEInspect.SetFocus
end;


function AddType(aNode: TTreeNode; Name: String; Info: PTypeInfo; aComponent:TComponent): TTreeNode;
begin
  Result:= fmIDEInspect.TreeView1.Items.AddChild(aNode, Name + ': ' + Info^.Name);
  Result.Data:= aComponent;
end;

Function LookupStuff : Boolean;
Var
  i, j, k, l: Integer;
Begin
  fmIDEInspect.TreeView1.items.clear;
  fmIDEInspect.TreeView1.Items.BeginUpdate;
  TopNode:=
    AddType(
      nil,
      Application.Name,
      Application.ClassInfo,
      Application);
  Try
  for i:=0 to Application.ComponentCount-1 do
    Begin
      Level1Child:=
        AddType(
          TopNode,
          Application.Components[i].Name,
          Application.Components[i].ClassInfo,
          Application.Components[i]);
      for j:=0 to Application.Components[i].ComponentCount-1 do
        begin
          Level2Child:=
            AddType(
              Level1Child,
              Application.Components[i].Components[j].Name,
              Application.Components[i].Components[j].ClassInfo,
              Application.Components[i].Components[j]);
          for k:=0 to Application.Components[i].Components[j].ComponentCount-1 do
            begin
              Level3Child:=
                AddType(
                  Level2Child,
                  Application.Components[i].Components[j].Components[k].Name,
                  Application.Components[i].Components[j].Components[k].ClassInfo,
                  Application.Components[i].Components[j].Components[k]);
              for l:=0 to Application.Components[i].Components[j].Components[k].ComponentCount-1 do
                begin
                  Level4Child:=
                    AddType(
                      Level3Child,
                      Application.Components[i].Components[j].Components[k].Components[l].Name,
                      Application.Components[i].Components[j].Components[k].Components[l].ClassInfo,
                      Application.Components[i].Components[j].Components[k].Components[l]);
                end;
            end;
        end;
      End;
  finally
  fmIDEInspect.TreeView1.Items.EndUpdate;
  End;
End;



{ TfmIDEInspect code }

procedure TfmIDEInspect.Button1Click(Sender: TObject);
begin
  CompEditForm.Execute(Self, False);
  LookupStuff;
end;

procedure TfmIDEInspect.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  aComponent: TComponent;
begin
  try
    with TreeView1 do
    begin
      aComponent:= Node.Data;
      CompEditForm.Execute(aComponent, False);
    end;
  finally
  end;
end;

procedure TfmIDEInspect.TreeView1Deletion(Sender: TObject; Node: TTreeNode);
begin
  Node.Data:= nil;
end;

procedure TfmIDEInspect.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CompEditForm.Close;
end;

end.
